==> inca_tool-1.0.0-py3-none-any.whl: inca/soilretention/preprocessing/fcover.py <==
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.


"""
SYNOPSIS
    fcover.py -i <folder_path> -o <folder_path> -y <year> -r <reference_file> [-v,--verbose] [-h,--help] [--version]

DESCRIPTION
    This snippet is for preprocessing of the FCOVER data out of the PROBAV or SENTINEL-3 archive.
    The average FCOVER of the year is calculated out of all input data in a folder. Then the raster is
    resampled plus warped to the needed AOI and resolution.
    Important: the 10daily input FCOVER data must be geotiffs (see lower comment for conversion).

PREREQUISITES
    Python => 3.7.0

EXAMPLES
    python -m inca.soilretention.preprocessing.fcover -i <folder_path> -o <folder_path> -y <year> -r <ref_file>

    OR

    python3 fcover.py -i <folder_path> -o <folder_path> -y <year> -r <ref_file>

EXIT STATUS
    Ctrl-C  =  cancel the script at any time

AUTHOR
    Dr. Marcel Buchhorn <marcel.buchhorn@vito.be>

VERSION
    1.2 (2023-03-02)
"""


'''
If you do not have 10-daily FCOVER data then you have to extract it from the CGLS:
- I use the cgls-geotiff package from VITO to access the right data and transform from netcdf to geotiff
1. clone the repository --> https://git.vito.be/scm/biggeo/cgls-geotiff.git
2. import it in folder on the MEP (e.g. ~/workspace/cgls_geotiff )
3. go to the root folder and do an editable install: python3 -m pip install --user -e .
4. check out the branch "fcover-inca"
5. now you can call in the terminal the processing of the nc files
e.g. python3 FCOVER_preparation_INCA.py BioPar_FCOVER_V2_Global 20000101 20001231 /data/inca_vol1/input_data/CGL_BioPar_FCOVER_V2_Global True
'''
################## Importing modules ##################
import logging
import optparse
import os
import time
import subprocess
import rasterio
import numpy as np
import inca
import inca.common.geoprocessing as geo
import inca.soilretention.preprocessing as preprocessing

############## END importing modules ##################

logger = logging.getLogger(preprocessing.__name__)


############## MAIN ###################################
def main():
    start_time = time.time()
    # parse the arguments
    options = parse_args()

    # ini the logging
    inca.set_up_console_logging(options.verbose)

    # change umask driver
    try:
        old_umask = os.umask(0o000)
    except Exception:
        logger.exception('Could not change UMASK setting')
        pass

    logger.info('START OF MODULE: INCA soil retention fcover preprocessing')

    # init the metadata and raster extent of project
    accord = geo.GeoProcessing("INCA-tool",
                               "Soil Retention pre-processing",
                               os.path.normpath(options.path_out),
                               ProjectExtentTiff=os.path.normpath(options.path_ref),
                               GDAL_verbose=options.verbose)
    os.makedirs(os.path.normpath(options.path_out), exist_ok=True)

    # run the average fcover generation
    logger.info('generating the average fcover in original resolution')
    try:
        path_fcover, fcover_dtype = processing_avg_fcover(options, accord.metadata)
    except Exception:
        logger.exception('Error during processing of the datasets to generate the average fcover')
        raise

    # run the warp to AOI
    logger.info('bring the final FCOVER product to AOI')
    try:
        path_out = os.path.join(os.path.normpath(options.path_out),
                                'CGLS_FCOVER_{}_annual-average_INCA_{}m_'
                                'EPSG{}.tif'.format(options.pYear,
                                                    int(accord.ref_profile['transform'].a),
                                                    accord.ref_profile['crs'].to_epsg()))
        accord.Warp2AOI(path_fcover, path_out, wResampling='near', secure_run=True, wOT=fcover_dtype)

    except Exception:
        logger.exception('Error during processing to bring dataset to final AOI and resolution')
        raise

    logger.info('All processing successfully done. Have a nice day.')
    # set umask to old settings
    try:
        os.umask(old_umask)
    except Exception:
        logger.exception('!!! Could not change UMASK setting back!!!')
        pass

    logger.debug('---------------------------')
    logger.debug('TOTAL TIME IN MINUTES: %10.4f', (time.time() - start_time) / 60)
    logger.debug('END OF MODULE: INCA soil retention fcover preprocessing')


def parse_args():
    parser = optparse.OptionParser(formatter=optparse.TitledHelpFormatter(), usage=globals()['__doc__'],
                                   version="%prog v1.2")
    parser.add_option('-v', '--verbose', action='store_true', default=False, help='verbose output')

    parser.add_option('-r', '--path_reference', help='Path to raster reference file for resampling. Needed.', dest='path_ref')
    parser.add_option('-i', '--path_in', help='Path to folder with FCOVER input data. Needed.', dest='path_in')
    parser.add_option('-o', '--path_out', help='Path to output folder. Needed.', dest='path_out')
    parser.add_option('-y', '--year', help='Processing year. Needed.', dest='pYear')

    # parse the given system arguments
    (options, args) = parser.parse_args()
    # do checks on the parsed options
    if (not options.path_ref) or (not os.path.isfile(os.path.normpath(options.path_ref))):
        parser.error("the -r argument for the reference raster is missing or the given path doesn't exist!!!")
    if (not options.path_in) or (not os.path.isdir(os.path.normpath(options.path_in))):
        parser.error("the -i argument for FCOVER input data is missing or the given path doesn't exist!!!")
    if (not options.pYear) or (not options.pYear.isdigit()):
        parser.error("the -y argument must be a year.")

    if len(args) != 0:
        parser.error('too many arguments')

    return options


def processing_avg_fcover(options, metadata, scale_factor=0.004, valid_range=[0, 250]):
    '''function to calculate the average FCOVER out of given files in folder
       NOTE: there is done no check - all tif files in folder are used to calculate average
       standard for CGLS 1km product
       valid_range: 0 - 250
       nodata:  251 - 255
       scaling: 0.004
       unit: fraction
       '''

    # we do not care about memory so we run all in one chunk
    # get list of all files in input folder
    lFiles = []
    for entry in os.scandir(os.path.normpath(options.path_in)):
        if entry.is_file() and entry.name.endswith(".tiff"):
            lFiles.append(entry.path)

    # get main profile data from first file
    with rasterio.open(lFiles[0]) as src:
        src_profile = src.profile
        try:
            s = float(src.tags(1)['scale_factor'])
            if (s >= 0) and (s <= 1.):
                scale_factor = s
        except BaseException:
            logger.info('use standard scale factor since no matching metadata.')
        try:
            v = [int(x) for x in (src.tags(1)['valid_range']).replace(']', ' ').split(' ') if x.isnumeric()]
            if (len(v) == 2) and (v[0] < v[1]):
                valid_range = v
        except BaseException:
            logger.info('use standard valid data range since no matching metadata.')

    # ini the arrays for sum and count
    aSum = np.zeros((src_profile['height'], src_profile['width']), dtype=np.uint)
    aCount = np.zeros_like(aSum, dtype=np.float)

    # no we loop over all files
    for element in lFiles:
        # get data
        with rasterio.open(element) as src:
            aData = src.read(1).astype(np.uint)
        # get valid data
        mValid = (aData >= valid_range[0]) & (aData <= valid_range[1])
        # fill prepared arrays
        aSum[mValid] += aData[mValid]
        aCount[mValid] += 1

    # now we calculate mean FCOVer value on Byte basis
    aMean = np.full_like(aSum, 255, dtype=np.ubyte)
    aMean[aCount > 0] = np.round(aSum[aCount > 0] / aCount[aCount > 0], 0)

    # setup dst profile
    src_profile.update(count=1, dtype=rasterio.ubyte, nodata=255, compress='deflate', tiled=True)

    # output filename
    path_out = os.path.join(os.path.normpath(options.path_out), 'CGLS_FCOVER_{}_annual-average.tif'.format(options.pYear))

    # setup tags
    metadata.read_raster_tags(lFiles)
    tags = metadata.prepare_raster_tags(f'Annual average of FCOVER derived from CGLS datasets for the '
                                        f'year {options.pYear}. value_range = [0, 250]. scaling_factor: {scale_factor}.'
                                        f'offset_factor=0', 'fraction')

    try:
        with rasterio.open(path_out, 'w', **src_profile) as dst:
            dst.update_tags(**tags)
            dst.scales = [scale_factor]
            dst.offsets = [0]
            dst.write(aMean.astype(src_profile['dtype']), 1)
    except BaseException:
        if os.path.exists(path_out):
            os.remove(path_out)
        raise IOError('generation of average annual FCOVER for year {} was not successful'.format(options.pYear))
    return path_out, src_profile['dtype']


if __name__ == '__main__':
    # hack to get the proper logger when running the script directly
    logger = logging.getLogger('inca.soilretention.preprocessing.fcover')
    main()

==> inca_tool-1.1.2-py3-none-any.whl: inca/soilretention/preprocessing/fcover.py <==
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.


"""
SYNOPSIS
    fcover.py -i <folder_path> -o <folder_path> -y <year> -r <reference_file> [-v,--verbose] [-h,--help] [--version]

DESCRIPTION
    This snippet is for preprocessing of the FCOVER data out of the PROBAV or SENTINEL-3 archive.
    The average FCOVER of the year is calculated out of all input data in a folder. Then the raster is
    resampled plus warped to the needed AOI and resolution.
    Important: the 10daily input FCOVER data must be geotiffs (see lower comment for conversion).

PREREQUISITES
    Python => 3.7.0

EXAMPLES
    python -m inca.soilretention.preprocessing.fcover -i <folder_path> -o <folder_path> -y <year> -r <ref_file>

    OR

    python3 fcover.py -i <folder_path> -o <folder_path> -y <year> -r <ref_file>

EXIT STATUS
    Ctrl-C  =  cancel the script at any time

AUTHOR
    Dr. Marcel Buchhorn <marcel.buchhorn@vito.be>

VERSION
    1.2 (2023-03-02)
"""


'''
If you do not have 10-daily FCOVER data then you have to extract it from the CGLS:
- I use the cgls-geotiff package from VITO to access the right data and transform from netcdf to geotiff
1. clone the repository --> https://git.vito.be/scm/biggeo/cgls-geotiff.git
2. import it in folder on the MEP (e.g. ~/workspace/cgls_geotiff )
3. go to the root folder and do an editable install: python3 -m pip install --user -e .
4. check out the branch "fcover-inca"
5. now you can call in the terminal the processing of the nc files
e.g. python3 FCOVER_preparation_INCA.py BioPar_FCOVER_V2_Global 20000101 20001231 /data/inca_vol1/input_data/CGL_BioPar_FCOVER_V2_Global True
'''
################## Importing modules ##################
import logging
import optparse
import os
import time
import subprocess
import rasterio
import numpy as np
import inca
import inca.common.geoprocessing as geo
import inca.soilretention.preprocessing as preprocessing

############## END importing modules ##################

logger = logging.getLogger(preprocessing.__name__)


############## MAIN ###################################
def main():
    start_time = time.time()
    # parse the arguments
    options = parse_args()

    # ini the logging
    inca.set_up_console_logging(options.verbose)

    # change umask driver
    try:
        old_umask = os.umask(0o000)
    except Exception:
        logger.exception('Could not change UMASK setting')
        pass

    logger.info('START OF MODULE: INCA soil retention fcover preprocessing')

    # init the metadata and raster extent of project
    accord = geo.GeoProcessing("INCA-tool",
                               "Soil Retention pre-processing",
                               os.path.normpath(options.path_out),
                               ProjectExtentTiff=os.path.normpath(options.path_ref),
                               GDAL_verbose=options.verbose)
    os.makedirs(os.path.normpath(options.path_out), exist_ok=True)

    # run the average fcover generation
    logger.info('generating the average fcover in original resolution')
    try:
        path_fcover, fcover_dtype = processing_avg_fcover(options, accord.metadata)
    except Exception:
        logger.exception('Error during processing of the datasets to generate the average fcover')
        raise

    # run the warp to AOI
    logger.info('bring the final FCOVER product to AOI')
    try:
        path_out = os.path.join(os.path.normpath(options.path_out),
                                'CGLS_FCOVER_{}_annual-average_INCA_{}m_'
                                'EPSG{}.tif'.format(options.pYear,
                                                    int(accord.ref_profile['transform'].a),
                                                    accord.ref_profile['crs'].to_epsg()))
        accord.Warp2AOI(path_fcover, path_out, wResampling='bilinear', secure_run=True, wOT=fcover_dtype)

    except Exception:
        logger.exception('Error during processing to bring dataset to final AOI and resolution')
        raise

    logger.info('All processing successfully done. Have a nice day.')
    # set umask to old settings
    try:
        os.umask(old_umask)
    except Exception:
        logger.exception('!!! Could not change UMASK setting back!!!')
        pass

    logger.debug('---------------------------')
    logger.debug('TOTAL TIME IN MINUTES: %10.4f', (time.time() - start_time) / 60)
    logger.debug('END OF MODULE: INCA soil retention fcover preprocessing')


def parse_args():
    parser = optparse.OptionParser(formatter=optparse.TitledHelpFormatter(), usage=globals()['__doc__'],
                                   version="%prog v1.2")
    parser.add_option('-v', '--verbose', action='store_true', default=False, help='verbose output')

    parser.add_option('-r', '--path_reference', help='Path to raster reference file for resampling. Needed.', dest='path_ref')
    parser.add_option('-i', '--path_in', help='Path to folder with FCOVER input data. Needed.', dest='path_in')
    parser.add_option('-o', '--path_out', help='Path to output folder. Needed.', dest='path_out')
    parser.add_option('-y', '--year', help='Processing year. Needed.', dest='pYear')

    # parse the given system arguments
    (options, args) = parser.parse_args()
    # do checks on the parsed options
    if (not options.path_ref) or (not os.path.isfile(os.path.normpath(options.path_ref))):
        parser.error("the -r argument for the reference raster is missing or the given path doesn't exist!!!")
    if (not options.path_in) or (not os.path.isdir(os.path.normpath(options.path_in))):
        parser.error("the -i argument for FCOVER input data is missing or the given path doesn't exist!!!")
    if (not options.pYear) or (not options.pYear.isdigit()):
        parser.error("the -y argument must be a year.")

    if len(args) != 0:
        parser.error('too many arguments')

    return options


def processing_avg_fcover(options, metadata, scale_factor=0.004, valid_range=[0, 250]):
    '''function to calculate the average FCOVER out of given files in folder
       NOTE: there is done no check - all tif files in folder are used to calculate average
       standard for CGLS 1km product
       valid_range: 0 - 250
       nodata:  251 - 255
       scaling: 0.004
       unit: fraction
       '''

    # we do not care about memory so we run all in one chunk
    # get list of all files in input folder
    lFiles = []
    for entry in os.scandir(os.path.normpath(options.path_in)):
        if entry.is_file() and entry.name.endswith(".tiff"):
            lFiles.append(entry.path)

    # get main profile data from first file
    with rasterio.open(lFiles[0]) as src:
        src_profile = src.profile
        try:
            s = float(src.tags(1)['scale_factor'])
            if (s >= 0) and (s <= 1.):
                scale_factor = s
        except BaseException:
            logger.info('use standard scale factor since no matching metadata.')
        try:
            v = [int(x) for x in (src.tags(1)['valid_range']).replace(']', ' ').split(' ') if x.isnumeric()]
            if (len(v) == 2) and (v[0] < v[1]):
                valid_range = v
        except BaseException:
            logger.info('use standard valid data range since no matching metadata.')

    # ini the arrays for sum and count
    aSum = np.zeros((src_profile['height'], src_profile['width']), dtype=np.uint)
    aCount = np.zeros_like(aSum, dtype=np.float)

    # no we loop over all files
    for element in lFiles:
        # get data
        with rasterio.open(element) as src:
            aData = src.read(1).astype(np.uint)
        # get valid data
        mValid = (aData >= valid_range[0]) & (aData <= valid_range[1])
        # fill prepared arrays
        aSum[mValid] += aData[mValid]
        aCount[mValid] += 1

    # now we calculate mean FCOVer value on Byte basis
    aMean = np.full_like(aSum, 255, dtype=np.ubyte)
    aMean[aCount > 0] = np.round(aSum[aCount > 0] / aCount[aCount > 0], 0)

    # setup dst profile
    src_profile.update(count=1, dtype=rasterio.ubyte, nodata=255, compress='deflate', tiled=True)

    # output filename
    path_out = os.path.join(os.path.normpath(options.path_out), 'CGLS_FCOVER_{}_annual-average.tif'.format(options.pYear))

    # setup tags
    metadata.read_raster_tags(lFiles)
    tags = metadata.prepare_raster_tags(f'Annual average of FCOVER derived from CGLS datasets for the '
                                        f'year {options.pYear}. value_range = [0, 250]. scaling_factor: {scale_factor}.'
                                        f'offset_factor=0', 'fraction')

    try:
        with rasterio.open(path_out, 'w', **src_profile) as dst:
            dst.update_tags(**tags)
            dst.scales = [scale_factor]
            dst.offsets = [0]
            dst.write(aMean.astype(src_profile['dtype']), 1)
    except BaseException:
        if os.path.exists(path_out):
            os.remove(path_out)
        raise IOError('generation of average annual FCOVER for year {} was not successful'.format(options.pYear))
    return path_out, src_profile['dtype']


if __name__ == '__main__':
    # hack to get the proper logger when running the script directly
    logger = logging.getLogger('inca.soilretention.preprocessing.fcover')
    main()
